#!/usr/bin/env python
#Rootnode Notify - Rootnode status notifier for GNOME
#Copyright (C) 2009 Damian Kulig <damian.kulig@gmail.com>
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.

import gobject
import gtk
import pynotify
import urllib2
import os.path

class Gui:

    ICON_NAME = "rootnode"
    ICON_NAME_ERROR = "rootnode_error"
    APP_NAME = "Rootnode monitor"
    APP_DESCRIPTION = "Rootnode status Notification Monitor for GNOME"
    WIN_BG_COLOR = "white"    #kolor tla okna, jako #HEX lub stala gtk.gdk.color_parse
    L_TXT_HIGH = "red"        #label font color if load = HIGH
    L_TXT_MED = "orange"      #label font color if load = MEDIUM
    L_TXT_LOW = "green"       #label font color if load = LOW
    WIN1_HOR_SIZE = 230     #szerokosc okna statusu
    WIN1_VER_SIZE = 100     #wysokosc okna statusu
    msg_temp = ""
    labels = []

    def __init__(self):
        pynotify.init("name")
        self._create_gui()
        self.linia = ""
        self.wartosci = "_init_"
        self.timeout_interval = 60000   #odpytywanie serwera co 60s   sek * 1000
        gobject.timeout_add(self.timeout_interval, self._rootnode_start, self)

    def _odpytaj_serwer(self):
        serwery = []
        for line in (urllib2.urlopen('http://rootnodestatus.net/status').read()).split()[:-1]:
          if line.count(":") == 3:  #standardowo 4 wartosci oddzielone 3 :
            serw, status, time, load = line.split(":");
            serwery.append((serw,status,time,load))
          else: #ae jesli jest trup to tylko dwie wartosci
            serw, status = line.split(":");
            serwery.append((serw, status, 0, 0))
        return serwery #linia[:-1]

    def _create_right_menu(self):
        self._rmenu = gtk.Menu();
        about = gtk.ImageMenuItem(stock_id=gtk.STOCK_ABOUT);
        about.connect("activate", self._on_about_clicked);
        quit = gtk.ImageMenuItem(stock_id=gtk.STOCK_QUIT);
        quit.connect("activate", self._on_exit_clicked);
        self._rmenu.add(about);
        self._rmenu.add(quit);
        self._rmenu.show_all();

    def _create_gui(self):        #piekno tez sie liczy
        #load themed or fallback app icon
        theme = gtk.icon_theme_get_default()
        if not theme.has_icon(self.ICON_NAME):
            theme.prepend_search_path(os.path.join(os.path.dirname(os.path.abspath(__file__)),'icons'))
        if theme.has_icon(self.ICON_NAME):
            self._icon_name = self.ICON_NAME
        if theme.has_icon(self.ICON_NAME_ERROR):
            self._icon_name_error = self.ICON_NAME_ERROR
        else:
            self._icon_name = gtk.STOCK_NETWORK
        #build tray icon
        self.tray = gtk.StatusIcon()
        self.tray.set_from_icon_name(self._icon_name)
        self.tray.connect('popup-menu', self._right_click) #prawy klik
        #obsluga click
        self.tray.connect("activate", self._on_click)
        self.tray.set_visible(True)
        
        self.window1 = ""
        self._create_right_menu()
        self._rootnode_start()
        self._kolejne_okno()

    def _set_tooltip(self, msg):
        gobject.idle_add(self.tray.set_tooltip, msg)

    def _right_click(self, status, button, time):
#        print "_on_right_click - right click on iconka :)";
        self._rmenu.popup(None, None, gtk.status_icon_position_menu, button, time, self.tray)
    
    def _list_to_str(self, lista):
      opis = ""
      for x in lista:
        serw, status, time, load = x
        opis += "%s\t%s\t%s\t%s\n" % (serw[0:3], status, time, load)
      return opis

    def _rootnode_start(self, msg="_init_"):
        self.lista = self._odpytaj_serwer();
        self.opis = self._list_to_str(self.lista)
        self._label_update()
        if "dead" in self.opis:                #zmienic na dead !!
          self.pynotify_msg = ""
          for x in self.lista:  # (serw, status, uptime, load)
            if "dead" in x:
              self.pynotify_msg +=str(x[0]+ "\t" + x[3] +"\tnie odpowiada\n")
            else:
              pass
          self.tray.set_from_icon_name(self._icon_name_error)
          self._ustaw_status(self.opis)
          self._powiadom(self.pynotify_msg)
        else:
          self._ustaw_status(self.opis)
          self.tray.set_from_icon_name(self._icon_name)
        return True #False if runonce

    def _powiadom(self, opis):
      if opis == self.msg_temp:
        return
      else:
        self.msg_temp = opis
        n = pynotify.Notification("Blad serwera", opis, "dialog-warning")
        n.set_urgency(pynotify.URGENCY_NORMAL)
        n.set_timeout(pynotify.EXPIRES_NEVER)
        n.show()

    def _ustaw_status(self, wartosci):
      if wartosci:
        self._set_tooltip(wartosci)
      else:
        self._set_tooltip("Error Connecting to Rootnode")

    def _kolejne_okno(self):
      self.window1 = gtk.Window(gtk.WINDOW_POPUP)#gtk.WINDOW_TOPLEVEL)
      self.window1.set_size_request(self.WIN1_HOR_SIZE, self.WIN1_VER_SIZE) #rozmiar okna
      self.window1.set_gravity(gtk.gdk.GRAVITY_SOUTH_EAST)
      self.window1.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(self.WIN_BG_COLOR))  #ustawiamy kolog tla dla okna
 
      width, height = self.window1.get_size()
      screen, rect, orient = self.tray.get_geometry()
      self.window1.move(gtk.gdk.screen_width() - width , rect.height/10+4)# rect.height ) #  height)
      
      self.window1.connect("delete_event", lambda w,e: gtk.main_quit())
      
      LINES = len(self.lista) #ile mamy serverow
      ROWS =  len(self.lista[0])  #ile mamy danych

#      self.labels = [] 
      self.grid = gtk.Table(LINES, ROWS,True)
      x = 0 
      for lines in self.lista:
        serv, live, up, load = lines
        l_serv = gtk.Label(serv)
        l_live = gtk.Label(live)
        l_up = gtk.Label(up)
        l_load = gtk.Label(load)
#        ebox = gtk.EventBox()
#        ebox.add(l_up)
#        self.grid.attach(ebox,2,3,x,x+1)
#        ebox.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse("blue"))
        self.grid.attach(l_serv,0,1,x,x+1)
        self.grid.attach(l_live,1,2,x,x+1)
        self.grid.attach(l_up,2,3,x,x+1)
        self.grid.attach(l_load,3,4,x,x+1)
        self.labels.append((l_serv, l_live, l_up, l_load))#, ebox))
        x +=1
      self.window1.add(self.grid)
      self.window1.show_all()
      self.window1.hide()
    
    def _label_update(self):  #aktualizacja caption dla label
      serw_limit = { "stallman":("10","25"), "cox":("30", "75"), \
                     "torvalds":("30", "75"), "farmer":("30", "75"), \
                     "venema":("25", "60"), "esr":("10", "20"), "korn":("8", "16")}
      for x, y in enumerate(self.labels):
        l_serv, l_live, l_up, l_load = y  #id dla label ,ebox
        serv, live, up, load = self.lista[x]  #dane serwerow
        l_serv.set_alignment(0.0,0.5)
        l_serv.set_justify(gtk.JUSTIFY_LEFT)
        if load < serw_limit[serv][0]:
          COLOR = '<span color ="' + self.L_TXT_LOW + '">%s</span>'
          l_serv.set_markup(COLOR % (serv))
          l_live.set_markup(COLOR % (live))
          l_up.set_markup(COLOR % (up))
          l_load.set_markup(COLOR % (load))
        elif load > serw_limit[serv][1]:
          COLOR = '<span color ="' + self.L_TXT_HIGH + '">%s</span>'
          l_serv.set_markup(COLOR % (serv))
          l_live.set_markup(COLOR % (live))
          l_up.set_markup(COLOR % (up))
          l_load.set_markup(COLOR % (load))
        else:
          COLOR = '<span color ="' + self.L_TXT_MED + '">%s</span>'
          l_serv.set_markup(COLOR  % (serv))
          l_live.set_markup(COLOR  % (live))
          l_up.set_markup(COLOR  % (up))
          l_load.set_markup(COLOR  % (load))

    def _on_click(self, status):
      if self.window1.get_property("visible"): #jesli okno jest widoczne to ukrywamy
         self.window1.hide()
      else:
        self.window1.show() #jesli okno ukryte pokazujemy

    def _on_about_clicked(self, widget):
      dlg = gtk.AboutDialog()
      dlg.set_name(self.APP_NAME)
      dlg.set_comments(self.APP_DESCRIPTION)
      dlg.set_authors(("Damian Kulig alias clayman\n#rootnode at irc.freenode.net",))
      dlg.set_logo_icon_name(self._icon_name)
      dlg.run()
      dlg.destroy()

    def _on_exit_clicked(self, widget):
      gtk.main_quit()

if __name__ == "__main__":
    gtk.gdk.threads_init()
    app = Gui()
    gtk.main()
